#define ANNEAL 10000     // number of annealing steps
#define NUMMIX 20    //maximum number of mixture components
#define LENADJ (HILL-LENITER)

char 	chosenmotif[NO_OF_SEQ][MOTIF_COUNT][MOTIFLENGTH];



/*#####################################################################*/
/*          Construction, manipulation of Data Structures              */
/*#####################################################################*/


void addmotifnode(motifnode *mylist, int newpos,char newmotif[MOTIFLENGTH]);
node *addnewnode(node *mylist, int newpos);   //adds a new starting position to the current list 
void addnode(node *mylist, int newpos);
void addpos(vect *myvect, int seq, int ps, int fr, char rof);
motifnode *copymotifs(int pos[MOTIF_COUNT], char newmotif[MOTIF_COUNT][MOTIFLENGTH],int total);
node *copynode(int pos[MOTIF_COUNT], int total);
table *maketable(node *oldlist);
int ReadSeq(FILE *fIn, FILE *fOut, char Seq[NoSeqs][MaxSeqLen], int SeqLen[NoSeqs]);
node *rmnode(node *mylist, int newpos);   //removes a starting position from the current l



/*#####################################################################*/
/*                   Initialization functions                          */
/*#####################################################################*/


void setmatzero(int mat[MOTIFLENGTH][NO_OF_BASES]);
void setveczero(int vec[DICT_SIZE], int n);



/*#####################################################################*/
/*                General functions: computation                       */
/*#####################################################################*/


int findindex(char b);  
int intmax(int array[NO_OF_BASES]);
int maximum(double array[MOTIFLENGTH],int n);
int maxposition(int array[NO_OF_SEQ][MOTIF_COUNT],int n[NO_OF_SEQ],int totseq);
int minposition(int array[NO_OF_SEQ][MOTIF_COUNT],int n[NO_OF_SEQ],int totseq);
int sample(double p[], int p_length);



/*#####################################################################*/
/*                General functions: comparison                        */
/*#####################################################################*/


int compare(int pos, node *startlist);
int criterion(double p[NO_OF_BASES][MOTIFLENGTH], int length, char newword[MOTIFLENGTH]);
int findmatch(int pos, node *mylist, int w);



/*#####################################################################*/
/*        Gamma and Dirichlet distribution-related functions           */
/*#####################################################################*/


double lngamma(double arg);
void rDirich(int k, double a[], double d[]);
void rgamma(double a, int n, double gam[]);
void Rgamma(double a, int n, double gam[]);



/*#####################################################################*/
/*          Insertion-deletion (gap)-related functions                 */
/*#####################################################################*/


struct Fmat FIND_ALIGNMENT(int pos, char SEQ[BIGSEQ],int wordno);
int FindFinalGaps(int pos, char SEQ[BIGSEQ],int wordno, int nogaps[MOTIFLENGTH]);
int FindGaps(int pos, char SEQ[BIGSEQ],int wordno, int nogaps[MOTIFLENGTH]);
struct Fmat FORWARD_SUM(int pos, char SEQ[BIGSEQ],int wordno);



/*#####################################################################*/
/*                  Metropolis-ratio calculations                      */
/*#####################################################################*/


double colratio(int den[NO_OF_SEQ][MOTIF_COUNT], int num[NO_OF_SEQ][MOTIF_COUNT], int d,char SEQ[NO_OF_SEQ][BIGSEQ]);
double endratio(int num[NO_OF_SEQ][MOTIF_COUNT],int den[NO_OF_SEQ][MOTIF_COUNT], int d,char SEQ[NO_OF_SEQ][BIGSEQ]);
double lengthprobratio(int matcount[NO_OF_BASES][MOTIFLENGTH], int count[NO_OF_BASES+2], double bb[NO_OF_BASES+2], double gam[NO_OF_BASES][MOTIFLENGTH],int incr, int chcol[NO_OF_BASES]);
void UpdateCount(int mat[NO_OF_BASES][MOTIFLENGTH], char motifstr[MOTIFLENGTH],int w);



/*#####################################################################*/
/*                String/motif manipulation                            */
/*#####################################################################*/


void CapitalizeMotif(char SEQ[BIGSEQ],char newSEQ[BIGSEQ], int pos);
void decreasemotif(char SEQ[NO_OF_SEQ][BIGSEQ],int newmatcount[NO_OF_BASES][MOTIFLENGTH],int count[NO_OF_BASES+2], int newcol[NO_OF_BASES], int posn[NO_OF_SEQ][MOTIF_COUNT], int ch);
void genmotif(double R[MOTIFLENGTH][NO_OF_BASES+1], int w, char motifstr[MOTIFLENGTH]);
void increasemotif(char SEQ[NO_OF_SEQ][BIGSEQ],int newmatcount[NO_OF_BASES][MOTIFLENGTH],int count[NO_OF_BASES+2], int newcol[NO_OF_BASES], int posn[NO_OF_SEQ][MOTIF_COUNT], int ch,double gam[NO_OF_BASES][MOTIFLENGTH]);
void motifplus(char motif[NO_OF_SEQ][MOTIF_COUNT][MOTIFLENGTH], char SEQ[NO_OF_SEQ][BIGSEQ], int end[NO_OF_SEQ][MOTIF_COUNT],int start[NO_OF_SEQ][MOTIF_COUNT]);
void motifminus(char motif[NO_OF_SEQ][MOTIF_COUNT][MOTIFLENGTH], char SEQ[NO_OF_SEQ][BIGSEQ], int start[NO_OF_SEQ][MOTIF_COUNT]);
void motifplusBF(char motif[NO_OF_SEQ][MOTIF_COUNT][MOTIFLENGTH], char SEQ[NO_OF_SEQ][BIGSEQ], int end[NO_OF_SEQ][MOTIF_COUNT],int start[NO_OF_SEQ][MOTIF_COUNT],int oldcounts[NO_OF_BASES+2]);
void motifminusBF(char motif[NO_OF_SEQ][MOTIF_COUNT][MOTIFLENGTH], char SEQ[NO_OF_SEQ][BIGSEQ], int start[NO_OF_SEQ][MOTIF_COUNT],int oldcounts[NO_OF_BASES+2]);
void newlengthmotifs(char SEQ[NO_OF_SEQ][BIGSEQ],int startposn[NO_OF_SEQ][MOTIF_COUNT],int endposn[NO_OF_SEQ][MOTIF_COUNT],int matcount[NO_OF_BASES][MOTIFLENGTH], int count[NO_OF_BASES+2], double bb[NO_OF_BASES], double gam[NO_OF_BASES][MOTIFLENGTH],int incr, int side);
void setMotif(char str[TOTLENGTH], char updateMotif[MOTIFLENGTH], int gapindices[MOTIFLENGTH]);
void setString(int i, int j, char str[], char newString[]);
void setWithGaps(char newstr[TOTLENGTH],char withgaps[MOTIFLENGTH],int gapindices[MOTIFLENGTH]);
void strLwr(char str1[], char str2[]);
void strUpr(char str1[], char str2[]);
void TakeComplement(char str[BIGSEQ], char newstr[BIGSEQ]);



/*#####################################################################*/
/*                        Probability calculations                     */
/*#####################################################################*/


int dimerProb(char str[]);
double FIND_BACK(int pos, char SEQ[BIGSEQ]);
double logProb(char str[], int word);
double prob_back(char base);   
double prob_mat(char base, int i, int word); 



/*#####################################################################*/
/*                        Printing structures                          */
/*#####################################################################*/


void MatPrint(struct Fmat F, int w, int gw);
void printlist(node *mylist);
void printmotifnode(motifnode *mylist,int seqno, FILE *fp, char SEQ[BIGSEQ]);
void printnode(node *mylist,int seqno, FILE *fp, char SEQ[BIGSEQ]);
void printout(vect *myvect);
void printpositions(node *mylist, FILE *fp,int seqno, char SEQ[BIGSEQ]);
int printvec(FILE *Fout, vect *myvect, char SEQ[NO_OF_SEQ][BIGSEQ],int LEN[NO_OF_SEQ],double Tfreq);



/*#####################################################################*/
/*          Likelihood, Bayes Factor, Significance test                */
/*#####################################################################*/


double backobslik(double probs[NO_OF_BASES],char SEQ[NO_OF_SEQ][BIGSEQ], int seqL[NO_OF_SEQ], int SeqNum);
double C2back(int w, double gam[NO_OF_BASES][MOTIFLENGTH], int D, int counts[DICT_SIZE], int countmat[NO_OF_BASES][MOTIFLENGTH]);
double ChiSqCutoff(int W, double cut);
double findlik(int w, double gam[NO_OF_BASES][MOTIFLENGTH], int D, int counts[DICT_SIZE], int countmat[DICT_SIZE][NO_OF_BASES][MOTIFLENGTH],int numwords);
double findlikSINGLE(int matcount[NO_OF_BASES][MOTIFLENGTH], int count[NO_OF_BASES], double bb[NO_OF_BASES], int sumcount, int Nmotifs, double gam[NO_OF_BASES][MOTIFLENGTH]);
double infoback(int mat[NO_OF_BASES][MOTIFLENGTH], int nw);
double infocalc(int mat[NO_OF_BASES][MOTIFLENGTH], double probmat[NO_OF_BASES][MOTIFLENGTH], int nw);
double KLdistance(int matcount[NO_OF_BASES][MOTIFLENGTH], double probs[NO_OF_BASES], double gam[NO_OF_BASES][MOTIFLENGTH], int w, int MotifNum);
double motifsig(int w, double gam[NO_OF_BASES][MOTIFLENGTH], int D, int counts[DICT_SIZE], int countmat[NO_OF_BASES][MOTIFLENGTH]);
double multmotifsig(int w, double gam[NO_OF_BASES][MOTIFLENGTH], int D, int counts[DICT_SIZE], int countmat[DICT_SIZE][NO_OF_BASES][MOTIFLENGTH],int numwords);
double p1theta(int w, int D, int counts[DICT_SIZE], int countmat[NO_OF_BASES][MOTIFLENGTH], double rho[DICT_SIZE], double theta[NO_OF_BASES][MOTIFLENGTH]);
double ratback(int w, int seq, int basecount[NO_OF_SEQ][NO_OF_BASES]);



/*#####################################################################*/
/*                       Program functions                             */
/*#####################################################################*/


void runBF();
void runLEN(FILE *f2, int numcomp);




